﻿using NetTopologySuite.Algorithm;
using NetTopologySuite.Geometries;
using NetTopologySuite.IO;
using NUnit.Framework;

namespace NetTopologySuite.Tests.NUnit.Algorithm
{
    [TestFixture]
    public class ComputeOrientationTest
    {
        private readonly WKTReader _reader = new WKTReader();

        [Test]
        public void TestCCW()
        {
            Assert.IsTrue(IsAllOrientationsEqual(GetCoordinates("LINESTRING ( 0 0, 0 1, 1 1)")));

            // experimental case - can't make it fail
            Coordinate[] pts2 = {
            new Coordinate(1.0000000000004998, -7.989685402102996),
            new Coordinate(10.0, -7.004368924503866),
            new Coordinate(1.0000000000005, -7.989685402102996),
            };
            Assert.IsTrue(IsAllOrientationsEqual(pts2));
        }

        // MD - deliberately disabled
        [Test]
        [Ignore("This case fails because subtraction of small from large loses precision")]
        public void TestBadCCW()
        {
            // this case fails because subtraction of small from large loses precision
            Coordinate[] pts1 = {
            new Coordinate(1.4540766091864998, -7.989685402102996),
            new Coordinate(23.131039116367354, -7.004368924503866),
            new Coordinate(1.4540766091865, -7.989685402102996),
            };
            Assert.IsTrue(IsAllOrientationsEqual(pts1));
        }

        private static bool IsAllOrientationsEqual(Coordinate[] pts)
        {
            var orient = new OrientationIndex[3];
            orient[0] = Orientation.Index(pts[0], pts[1], pts[2]);
            orient[1] = Orientation.Index(pts[1], pts[2], pts[0]);
            orient[2] = Orientation.Index(pts[2], pts[0], pts[1]);
            return orient[0] == orient[1] && orient[0] == orient[2];
        }

        private Coordinate[] GetCoordinates(string wkt)
        {
            var geom = _reader.Read(wkt);
            return geom.Coordinates;
        }
    }
}
